@@ -144,7 +144,8 @@ sub algorithm
 		$self->set_tag("a", shift);
 	}
 
-	return lc $self->get_tag("a");
+	my $a = $self->get_tag("a");
+	return defined $a ? lc $a : undef;
 }	
 
 =head2 as_string() - the signature header as a string
@@ -164,7 +165,7 @@ sub as_string
 {
 	my $self = shift;
 
-	my $prefix = $self->{prefix} || "DKIM-Signature:";
+	my $prefix = $self->{prefix} || $self->DEFAULT_PREFIX;
 
 	return $prefix . $self->SUPER::as_string;
 }
@@ -174,7 +175,7 @@ sub as_string_debug
 {
 	my $self = shift;
 
-	my $prefix = $self->{prefix} || "DKIM-Signature:";
+	my $prefix = $self->{prefix} || $self->DEFAULT_PREFIX;
 
 	return $prefix . join(";", map { ">" . $_->{raw} . "<" } @{$self->{tags}});
 }
@@ -281,7 +282,8 @@ sub canonicalization
 		$self->set_tag("c", join("/", @_));
 	}
 
-	my $c = lc $self->get_tag("c");
+	my $c = $self->get_tag("c");
+	$c = lc $c  if defined $c;
 	if (not $c)
 	{
 		$c = "simple/simple";
@@ -289,8 +291,8 @@ sub canonicalization
 	my ($c1, $c2) = split(/\//, $c, 2);
 	if (not defined $c2)
 	{
-		# default body canonicalization depends on header canonicalization
-		$c2 = $c1 eq "nowsp" ? "nowsp" : "simple";
+		# default body canonicalization is "simple"
+		$c2 = "simple";
 	}
 
 	if (wantarray)
@@ -407,7 +409,8 @@ sub decode_qp
 	my $res = shift;
 
 	#TODO- should I worry about non-ASCII systems here?
-	$res =~ s/=([\da-fA-F]{2})/pack("C", hex($1))/ge;
+	$res =~ s/=([\da-fA-F]{2})/pack("C", hex($1))/ge
+		if defined $res;
 	return $res;
 }
 
@@ -422,10 +425,16 @@ sub encode_qp
 	my $DISALLOWED = qr/[^!"#\$%&'()*+,\-.\/0-9:;<>?\@A-Z[\\\]^_`a-z{|}~]/;
 
 	#TODO- should I worry about non-ASCII systems here?
-	$res =~ s/($DISALLOWED)/sprintf('=%02X', ord($1))/eg;
+	$res =~ s/($DISALLOWED)/sprintf('=%02X', ord($1))/eg
+		if defined $res;
 	return $res;
 }
 
+sub DEFAULT_PREFIX
+{
+	return "DKIM-Signature:";
+}
+
 =head2 domain() - get or set the domain (d=) field
 
   my $d = $signature->domain;          # gets the domain value
@@ -449,7 +458,8 @@ sub domain
 		$self->set_tag("d", shift);
 	}
 
-	return lc $self->get_tag("d");
+	my $d = $self->get_tag("d");
+	return defined $d ? lc $d : undef;
 }	
 
 =head2 expiration() - get or set the signature expiration (x=) field
@@ -485,6 +495,53 @@ sub get_algorithm_class
 	return $class;
 }
 
+# [private method]
+# fetch_public_key() - initiate a DNS query for fetching the key
+#
+# This method does NOT return the public key.
+# Use get_public_key() for that.
+#
+sub fetch_public_key
+{
+	my $self = shift;
+	return if exists $self->{public_key_query};
+
+	my $on_success = sub {
+			if ($_[0]) {
+				$self->{public} = $_[0];
+			} else {
+				$self->{public_error} = "not available\n";
+			}
+		};
+
+	$self->{public_key_query} =
+		Mail::DKIM::PublicKey->fetch_async(
+			Protocol => $self->protocol,
+			Selector => $self->selector,
+			Domain => $self->domain,
+			Callbacks => {
+			Success => $on_success,
+			Error => sub { $self->{public_error} = shift },
+			},
+			);
+	return;
+}
+
+#EXPERIMENTAL
+sub _refetch_public_key
+{
+	my $self = shift;
+	if ($self->{public_key_query})
+	{
+		# clear the existing query by waiting for it to complete
+		$self->{public_key_query}->();
+	}
+	delete $self->{public_key_query};
+	delete $self->{public};
+	delete $self->{public_error};
+	$self->fetch_public_key;
+}
+
 =head2 get_public_key() - fetches the public key referenced by this signature
 
   my $pubkey = $signature->get_public_key;
@@ -503,30 +560,27 @@ sub get_public_key
 {
 	my $self = shift;
 
-	unless (exists $self->{public})
+	# this ensures we only try fetching once, even if an error occurs
+	if (not exists $self->{public_key_query})
 	{
-		# this ensures we only try fetching once, even if an error occurs
-		$self->{public} = undef;
+		$self->fetch_public_key;
+	}
 
-		my $pubk = Mail::DKIM::PublicKey->fetch(
-			Protocol => $self->protocol,
-			Selector => $self->selector,
-			Domain => $self->domain);
-		unless ($pubk)
-		{
-			die "not available\n";
-		}
+	if ($self->{public_key_query})
+	{
+		# wait for public key query to finish
+		$self->{public_key_query}->();
+		$self->{public_key_query} = 0;
+	}
 
-		if ($pubk->revoked)
-		{
-			# FIXME- the key was checked in fetch(), so if the
-			# key was really revoked, we shouldn't have gotten here
-			die "revoked\n";
-		}
-
-		$self->{public} = $pubk;
+	if (exists $self->{public})
+	{
+		return $self->{public};
+	}
+	else
+	{
+		die $self->{public_error};
 	}
-	return $self->{public};
 }
 
 =head2 get_tag() - access the raw value of a tag in this signature
@@ -670,12 +724,20 @@ sub identity_matches
 	return lc($addr) eq lc($id);
 }
 
-=head2 key() - get or set the key object
+=head2 key() - get or set the private key object
 
   my $key = $signature->key;
 
   $signature->key(Mail::DKIM::PrivateKey->load(File => "private.key"));
 
+The private key is used for signing messages.
+It is not used for verifying messages.
+
+The key object can be any object that implements the
+L<sign_digest()|Mail::DKIM::PrivateKey/"sign_digest()"> method.
+(Providing your own object can be useful if your actual keys
+are stored out-of-process.)
+
 =cut
 
 sub key
@@ -760,6 +822,9 @@ sub result
 
   my $detail = $signature->result_detail;
 
+An explanation of possible detail messages can be found in the
+documentation for L<Mail::DKIM::Verifier/result_detail()>.
+
 =cut
 
 sub result_detail
@@ -805,7 +870,7 @@ sub prettify
 {
 	my $self = shift;
 	$self->wrap(
-		Start => length($self->{prefix} || "DKIM-Signature:"),
+		Start => length($self->{prefix} || $self->DEFAULT_PREFIX),
 		Tags => {
 			b => "b64",
 			bh => "b64",
@@ -827,7 +892,7 @@ sub prettify_safe
 {
 	my $self = shift;
 	$self->wrap(
-		Start => length($self->{prefix} || "DKIM-Signature:"),
+		Start => length($self->{prefix} || $self->DEFAULT_PREFIX),
 		Tags => {
 			b => "b64",
 			},
