@@ -10,9 +10,8 @@
 use strict;
 use warnings;
 
-use Mail::DKIM::Algorithm::rsa_sha1;
+use Mail::DKIM::PrivateKey;
 use Mail::DKIM::Signature;
-use Mail::Address;
 
 =head1 NAME
 
@@ -21,6 +20,7 @@ Mail::DKIM::Signer - generates a DKIM signature for a message
 =head1 SYNOPSIS
 
   use Mail::DKIM::Signer;
+  use Mail::DKIM::TextWrap;  #recommended
 
   # create a signer object
   my $dkim = Mail::DKIM::Signer->new(
@@ -48,10 +48,33 @@ Mail::DKIM::Signer - generates a DKIM signature for a message
 
   # what is the signature result?
   my $signature = $dkim->signature;
+  print $signature->as_string;
+
+=head1 DESCRIPTION
+
+This class is the part of L<Mail::DKIM> responsible for generating
+signatures for a given message. You create an object of this class,
+specifying the parameters of the signature you wish to create, or
+specifying a callback function so that the signature parameters can
+be determined later. Next, you feed it the entire message using
+L</"PRINT()">, completing with L</"CLOSE()">. Finally, use the
+L</"signatures()"> method to access the generated signatures.
+
+=head2 Pretty Signatures
+
+L<Mail::DKIM> includes a signature-wrapping module (which inserts
+linebreaks into the generated signature so that it looks nicer in the
+resulting message. To enable this module, simply call
+
+  use Mail::DKIM::TextWrap;
+
+in your program before generating the signature.
 
 =head1 CONSTRUCTOR
 
-=head2 new() - construct an object-oriented signer
+=head2 new()
+
+Construct an object-oriented signer.
 
   # create a signer using the default policy
   my $dkim = Mail::DKIM::Signer->new(
@@ -97,7 +120,7 @@ L<Mail::DKIM::PrivateKey> object.
 package Mail::DKIM::Signer;
 use base "Mail::DKIM::Common";
 use Carp;
-our $VERSION = 0.32;
+our $VERSION = 0.38;
 
 # PROPERTIES
 #
@@ -287,7 +310,9 @@ sub finish_body
 
 =head1 METHODS
 
-=head2 PRINT() - feed part of the message to the signer
+=head2 PRINT()
+
+Feed part of the message to the signer.
 
   $dkim->PRINT("a line of the message\015\012");
 
@@ -295,14 +320,24 @@ Feeds content of the message being signed into the signer.
 The API is designed this way so that the entire message does NOT need
 to be read into memory at once.
 
-=head2 CLOSE() - call this when finished feeding in the message
+Please note that although the PRINT() method expects you to use
+SMTP-style line termination characters, you should NOT use the
+SMTP-style dot-stuffing technique described in RFC 2821 section 4.5.2.
+Nor should you use a <CR><LF>.<CR><LF> sequence to terminate the
+message.
+
+=head2 CLOSE()
+
+Call this when finished feeding in the message.
 
   $dkim->CLOSE;
 
 This method finishes the canonicalization process, computes a hash,
 and generates a signature.
 
-=head2 add_signature() - used by signer policy to create a new signature
+=head2 add_signature()
+
+Used by signer policy to create a new signature.
 
   $dkim->add_signature(new Mail::DKIM::Signature(...));
 
@@ -328,7 +363,9 @@ sub add_signature
 	return;
 }
 
-=head2 algorithm() - get or set the selected algorithm
+=head2 algorithm()
+
+Get or set the selected algorithm.
 
   $alg = $dkim->algorithm;
 
@@ -346,7 +383,9 @@ sub algorithm
 	return $self->{Algorithm};
 }
 
-=head2 domain() - get or set the selected domain
+=head2 domain()
+
+Get or set the selected domain.
 
   $alg = $dkim->domain;
 
@@ -364,7 +403,9 @@ sub domain
 	return $self->{Domain};
 }
 
-=head2 load() - load the entire message from a file handle
+=head2 load()
+
+Load the entire message from a file handle.
 
   $dkim->load($file_handle);
 
@@ -374,7 +415,9 @@ terminators (same as the SMTP protocol).
 
 =cut
 
-=head2 headers() - determine which headers to put in signature
+=head2 headers()
+
+Determine which headers to put in signature.
 
   my $headers = $dkim->headers;
 
@@ -426,12 +469,19 @@ sub want_header
 	return scalar grep { lc($_) eq lc($header_name) } @DEFAULT_HEADERS;
 }
 
-=head2 key() - get or set the private key object
+=head2 key()
+
+Get or set the private key object.
 
   my $key = $dkim->key;
 
   $dkim->key(Mail::DKIM::PrivateKey->load(File => "private.key"));
 
+The key object can be any object that implements the
+L<sign_digest() method|Mail::DKIM::PrivateKey/"sign_digest()">.
+(Providing your own object can be useful if your actual keys
+are stored out-of-process.)
+
 If you use this method to specify a private key,
 do not use L</"key_file()">.
 
@@ -448,7 +498,9 @@ sub key
 	return $self->{Key};
 }
 
-=head2 key_file() - get or set the filename containing the private key
+=head2 key_file()
+
+Get or set the filename containing the private key.
 
   my $filename = $dkim->key_file;
 
@@ -470,7 +522,9 @@ sub key_file
 	return $self->{KeyFile};
 }
 
-=head2 method() - get or set the selected canonicalization method
+=head2 method()
+
+Get or set the selected canonicalization method.
 
   $alg = $dkim->method;
 
@@ -488,7 +542,9 @@ sub method
 	return $self->{Method};
 }
 
-=head2 message_originator() - access the "From" header
+=head2 message_originator()
+
+Access the "From" header.
 
   my $address = $dkim->message_originator;
 
@@ -504,7 +560,9 @@ To get just the email address part, do:
 
 See also L</"message_sender()">.
 
-=head2 message_sender() - access the "From" or "Sender" header
+=head2 message_sender()
+
+Access the "From" or "Sender" header.
 
   my $address = $dkim->message_sender;
 
@@ -526,7 +584,9 @@ the "originator" would be the actual author.
 
 =cut
 
-=head2 selector() - get or set the current key selector
+=head2 selector()
+
+Get or set the current key selector.
 
   $alg = $dkim->selector;
 
@@ -544,19 +604,23 @@ sub selector
 	return $self->{Selector};
 }
 
-=head2 signature() - access the generated signature object
+=head2 signature()
+
+Access the generated signature object.
 
   my $signature = $dkim->signature;
 
 Returns the generated signature. The signature is an object of type
-Mail::DKIM::Signature. If multiple signatures were generated, this method
+L<Mail::DKIM::Signature>. If multiple signatures were generated, this method
 returns the last one.
 
-The signature should be B<prepended> to the message to make the
+The signature (as text) should be B<prepended> to the message to make the
 resulting message. At the very least, it should precede any headers
 that were signed.
 
-=head2 signatures() - access list of generated signature objects
+=head2 signatures()
+
+Access list of generated signature objects.
 
   my @signatures = $dkim->signatures;
 
